import os, sys, time
import json, re
from queue import Queue, Empty
from termcolor import cprint
from urllib.parse import urljoin
from functools import wraps

sys.path.append(os.getenv("Q_PATH"))
from qlib.asyn import Exe, futures
from qlib.net import to
from qlib.graphy import random_choice
from qlib.spide.agents import AGS
from qlib.log import LogControl

LogControl.LOG_LEVEL = LogControl.OK
LogControl.LOG_LEVEL |= LogControl.WRN
LogControl.LOG_LEVEL |= LogControl.INFO
LogControl.LOG_LEVEL |= LogControl.FAIL

info = LogControl.info
err = LogControl.err
fail = LogControl.fail
ok = LogControl.ok
wrn = LogControl.wrn

try:
    from lxml.etree import HTML as _Parser
except ImportError :
    cprint("no lxml", "yellow")
    try:
        from bs4 import BeautifulSoup as _Parser
    except ImportError:
        cprint("no beautifulSoup", "red")
        sys.exit(0)


class HTTPError(Exception):
    """
    a base class for HTTP error
    """
    def __init__(self, msg, code):
        super().__init__(msg, code)
        self.code = code
        self.msg = msg

    def __str__(self):
        return "HTTP Error(%d): %s " % (self.code, self.msg)

class Error404(HTTPError):
    pass

class UrlValid(HTTPError):
    pass


def after(func):
    @wraps(func)
    def _after(futures):
        return func(futures.result())




class Handler:
    
    back_executor = None
    
    def __init__(self, thread_num=12, **kargs):
        self.host = None
        self.start_url = None
        self.domain = None

        self.unreadch_url = Queue()
        self.try_times = 10
        self.reach_url = set()
        self.stop = False
        self.outer_url = set()
        self.mail_url = set()
        self.js_url = set()
        self.all_links = set()

        self.charset = None
        
        self.db = None
        self.stop_count = 0
        self.parser = _Parser
        self.grep = None
        if not self.back_executor:
            Handler.back_executor = Exe(thread_num)

        # init args
        for k in kargs:
            if hasattr(self, k):
                setattr(self, k, kargs[k])

        if not self.start_url:
            raise UrlValid("not set start url", 0)
            sys.exit(0)

        if not self.domain:

            self.domain = self.start_url.split("/")[2]
            info("init domain :", self.domain)

        if not self.host:
            self.host = '/'.join(self.start_url.split("/")[:3])
            info("init host :", self.host)

        self._index(self.on_start)

    
    def _index(self, call):
        self.Got(self.start_url, call)
        

    def _after_deal(self, url):
        if not url:
            return
        if self.domain in url:
            self.reach_url.add(url)
        else:    
            self.outer_url.add(url)
        

    def ready_link(self, url):
        if url is None:
            return
        # info(url, len(url))
        url = url.strip()
        if url.startswith("None"):
            return
        if url.startswith("javascript"):
            self.js_url.add(url)
            return

        if url.startswith("#"):
            return

        if url.startswith("mailto:"):
            self.mail_url.add(url)
            return


        if url.startswith("http"):
            if self.domain in url.split("/")[2]:
                self.unreadch_url.put(url)
                self.all_links.add(url)
        elif url.startswith("/"):
            self.unreadch_url.put(urljoin(self.host, url))
            self.all_links.add(urljoin(self.host, url))
        elif url.startswith("www"):
            u = 'http://' + url
            if self.domain in  url.split("/")[0]:
                self.unreadch_url.put(u)
                self.all_links.add(u)
        else:
            pass
            # LogControl.err('ord :', url)

    def filter(self, url):
        return re.search(self.grep, url)

    def get_link(self):
        try:
            u = self.unreadch_url.get(timeout=120)
            if not u in self.reach_url:
                return u
        except Empty:
            cprint("long time no link , auto exited !", "green")
            sys.exit(0)

    def Got(self,real_url, after_done):
        """
        do a request from real_url, deal response in after_done asynchronously
        """
        Handler.back_executor.done(self._link, after_done, real_url)
        self._after_deal(real_url)

    def _link(self, url, encoding='utf8'):
        try:
            agent = random_choice(AGS)
            res = to(url, headers={'user-agent':agent})
            if res.status_code != 200:
                raise Error404("not reachable", res.status_code)
            if self.charset:
                return url, self.parser(res.content.decode(self.charset, 'ignore'))
            return url, self.parser(res.content.decode(encoding, 'ignore'))
        except HTTPError as e:
            LogControl.fail(url, e.code)
            return url, None
        except Exception as e:
            LogControl.wrn(url)
            LogControl.err(url, e, txt_color='yellow')
            return url, None
        else:
            LogControl.ok(url)

    def pause(self):
        self.stop = True

    def dead(self):
        self.stop = True
        self.unreadch_url = Queue()

    def run(self, callback, limit_wait=10):
        try:
            self.stop = False
            while not self.stop:
                if self.unreadch_url.empty():
                    info("\t\twait target pull", self.stop_count, end='\r')
                    time.sleep(1)
                    self.stop_count += 1
                    if self.stop_count > limit_wait:
                        self.stop = True
                    continue

                link = self.get_link()
                if not link:
                    continue
                self.Got(link, callback)

                if self.stop_count > limit_wait:
                    self.stop = True
            print()
        finally:
            self.stop = True
            self.stop_count = 0

    def to_db(self, k, v):
        self.db.set(k, v)

    def on_start(self, futures):
        raise NotImplementedError()

    def on_detail(self, futures):
        raise NotImplementedError()



import os
import sys
from  os.path import join as j

from qlib.c import include, C, c, c_long, c_short,  c_ushort, c_char, c_char_p, c_int, c_ubyte, Structure, POINTER
from qlib.c import structure, c_restype

class LibStructure:
    """
    """

    @structure(1024)
    class py_ip_pack(C):
        _raw = """
    typedef struct ip_packet{
        long ip_len;
        long payload_len;
        char src[16] ;
        char dst[16] ;
        u_char protocol;
        u_char payload[1500];
    } py_ip_pack; 
    """
        pass


    @structure(1024)
    class py_tcp(C):
        """docstring for Tcp"""
        _raw = """
typedef struct tcp_packet{
    long len;
    long payload_len;
    unsigned short sport;
    unsigned short dport;
    unsigned int seq;
    unsigned int ack;
    unsigned int off;
    u_char flags;
    unsigned short checksum;
    unsigned short windows;
    u_char payload[1500];
} py_tcp;

        """
        pass


    @structure(1024)
    class py_udp(C):
        _raw = """

    typedef struct udp_packet{
        unsigned short sport;
        unsigned short dport;
        unsigned short ulen;
        unsigned short checksum;
        u_char * payload;
    }py_udp;
        """

# class Packet(C):
#     _fields_ = [
#         ()
#     ]


# this decorator will auto load lib
qlib_c_lib_path = j(os.getenv("HOME"), ".Q/clib/")


@include(qlib_c_lib_path, LibStructure)
class libsniff:
    """
    黑魔法的应用例子  , 三个函数 @structure, @include , @c
    
    structure :int ， 这个参数是指明 u_char * 长度上限的。。。不加容易崩溃，当常量吧
    创建一个c 中的同名类，继承 C， 然后直接把 结构体粘贴过来
    放到 _raw 变量里 。好了，c结构体转 py完成

    如下：
    @structure(1024)
    class py_udp(C):
        _raw = \"\"\"
    typedef struct udp_packet{
        unsigned short sport;
        unsigned short dport;
        unsigned short ulen;
        unsigned short checksum;
        u_char * payload;
    }py_udp;
    \"\"\"

    include str: lib path , obj: a place kan get structures
    创建一个和 so lib库同名的类 ， 第一个参数是 c 库的目录 ，（因为我偷懒， 必须把头文件和 so文件都放在这)
    第二个 变量 随便一个 obj 都行 ，随便放，只要 包含需要用到的 结构体类 ，结构体类的创建就上上面那个，非常傻瓜的方式

    
    c
    这个函数很黑，
    转换py的基础结构到c的结构，这里的基础结构，指凡是 ctypes能对应到 c的py结构都行
    比如 str -> c_char_array , int -> c_int , 
    复合结构也能自动转换 [[2,3], [3,4]] -> c_int_array_2_int_array_2 

    """

    @c
    def __init__(self, dev):
        libsniff.init_pcap(dev)

    @c
    def ip_pack(self, filter):
        # self.next_ip_pack.restype = POINTER(Ip)
        return self.next_ip_pack(filter)

    @c
    def tcp(self,cmd):
        ip = self.ip_pack(cmd)
        if ip.contents.protocol == 6:
            return self.to_tcp(ip)
        else:
            return self.tcp(cmd)

    @c
    def udp(self, cmd):
        ip = self.ip_pack(cmd)
        if ip.contents.protocol == 17:
            return self.to_udp(ip)
        elif ip.contents.protocol == 1:
            print("icmp:",ip.src,'->',ip.dst)
        else:
            return self.udp(cmd)

    @c
    def one(self, cmd):
        ip = self.ip_pack(cmd)
        if ip.contents.protocol == 6:
            t = self.to_tcp(ip)
            t.src = ip.contents.src.decode("utf8","ignore").strip()
            t.dst = ip.contents.dst.decode("utf8","ignore").strip()
            return t
        elif ip.contents.protocol == 17:
            u = self.to_udp(ip)
            u.src = ip.contents.src.decode("utf8","ignore").strip()
            u.dst = ip.contents.dst.decode("utf8","ignore").strip()
            return u
        else:
            ip.src = ip.contents.src.decode("utf8","ignore").strip()
            ip.dst = ip.contents.dst.decode("utf8","ignore").strip()
            return ip

    @c
    def mul(self, cmd):
        count = 0
        try:
            while True:
                count += 1
                yield self.one(cmd)
        
        except KeyboardInterrupt:
            print(count)


    def payload(self, cmd, pre_func):
        for p in self.mul(cmd):
            if p.contents.payload_len == 0:
                continue
            pay = pre_func((p.src, p.dst), p.contents.payload[:p.contents.payload_len])
            yield pay





import requests as R
import urllib
import sys
import json
import logging

try:
    from qmongohelper import Mongo
except Exception as  e:
    logging.warn("you should install 'qmongo_heler', this  you can found in github")
else:
    pass
finally:
    pass
class Base:
    main_url = 'https://api.zoomeye.org'
    urls = {
        'login': '/user/login',
        'host': 'GET /host/search',
    }


class Zoomeye(Base):

    def __init__(self, u, p):
        self.u = u
        self.p = p
        self.session = R.Session()
        self.token = json.loads(self.session.post(Base.main_url+Base.urls['login'],
                                        data=json.dumps({
                                            'username':u,
                                            'password':p})).content.decode("utf8"))['access_token']
        self.session.headers['Authorization'] = 'JWT ' + self.token
        self.headers = self.session.headers
    
    def _u(self, url):
        return urllib.parse.urljoin(Base.main_url, url) 

    def _read_config(self, label):
        method_type, url = Base.urls[label].split()
        return getattr(self.session, method_type.lower()), self._u(url)

    def _tr(self, dic):
        return '&'.join([ '='.join([pair[0], str(pair[1])])  for pair in  dic.items() ])

    def scan(self, typ, **kargs):
        func, url = self._read_config(typ)
        args = self._tr(kargs)
        logging.info(args)
        # print(url,args)
        url = url + '' if args.endswith('&') else url + '?' #, headers=self.headers)
        logging.debug(url  + args)
        return func(url + args).json()

class Scanner(Zoomeye):

    def __init__(self, *args, db=None, **kargs):
        super(Zoomeye,self).__init__(*args, **kargs)
        if db:
            self.db = dbheler(db)

    def hosts(self, **kargs):
        res_json = super(Scaner, self).scan("host", **kargs)
        self.db.insert("hosts", **res_json)


def main():
    pass
if __name__ == '__main__':
    main()



import time
from contextlib import contextmanager
from termcolor import cprint

from ._pipe import stdout

_do_some = lambda x: cprint("%s ..." % x, "yellow", end="")
_ok = lambda : cprint("\b\b\b ok", "green", attrs=['bold'])
_err = lambda : cprint("\b\b\b err", "red", attrs=['bold'])


@contextmanager
def just_info(info):
    try:
        _do_some(info)
        with stdout(None):
            yield
    except:
        _err()
    else:
        _ok()



from .text import __text as text
from .net import __urls as urls
from .asyn import __exp as asyn


__all__ = [
    'text',
	'urls',
	'asyn',
    'gen_random_serise',
]


def gen_random_serise(max_len=7):
    import random
    import os
    count = 0
    u = ''
    while count < max_len + random.randint(3,7):
        c = os.urandom(1).decode('utf8','ignore').strip()
        try:
            ch = ord(c)
        except:
            continue
        if  (ch >= 48 and ch <= 57) or (ch >= 65 and ch <= 90) or (ch >= 97 and ch <= 122):
            # print(c)
            u += c
            count += 1
    return u


import time
import datetime
from hashlib import sha256


def _hash(obj):
    return sha256(obj.encode('utf8')).hexdigest()



class SqlEngine:


    DEFAULT_TABLE = """CREATE TABLE %s (
        ID INTEGER PRIMARY KEY NOT NULL AUTO_INCREMENT ,
        CreatedTime TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP, 
        {extends} );"""

    def __init__(self,database=None, type='sqlite' , **connect_options):
        self.Type = type
        engine = None
        if not database and type == 'sqlite':
            type = 'mysql'
            self.Type = 'mysql'
            database = 'test'

        try:
            if type == 'sqlite':
                self.DEFAULT_TABLE = self.DEFAULT_TABLE.replace('AUTO_INCREMENT', '')
                import sqlite3
                engine = sqlite3
            elif type == 'mysql':
                import pymysql
                engine = pymysql
            elif type == 'postgresql':
                import psycopg2
                engine = psycopg2
            else:
                raise Exception("un supported sql: %s" % type)

        except ImportError as e:
            print(e)
        if self.Type == 'sqlite':
            self.con = engine.connect(database, **connect_options)
        else:

            self.con = engine.connect(database=database, **connect_options)
        self.cu = self.con.cursor()

    def got_datetime(timeobj):
        if isinstance(timeobj, datetime.datetime):
            return timeobj
        elif isinstance(timeobj, str):
            ss = time.strptime(timeobj, '%Y-%m-%d %H:%M:%S')
            return datetime.datetime.fromtimestamp(time.mktime(ss))

    def run_cmd(self, cmd):
        try:
            self.cu.execute(cmd)
        except Exception as e:
            print(e)
            print(cmd)

        return self.con.commit()

    def create_table(self, table, *extends_columns):
        columns = ',\n\t'.join(extends_columns)
        cmd = self.DEFAULT_TABLE.format(extends=columns) % table
        self.run_cmd(cmd)
        return cmd

    def create(self, table, extend_options={}, **kargs):
        """
        this function can create table smarter:
        example:
            create('User', name=str, login_time=time, passwd=hash, own=1, number=int, id_f=Table)
            will ->  CREATE TABLE User (
                            ID INTEGER PRIMARY KEY NOT NULL [AUTO_INCREMENT] ,
                            CreatedTime TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                            name TEXT NOT NULL,
                            login_time TimeStamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                            passwd text, # hash(str),
                            own INTEGER NOT NULL DEFAULT 1 ,
                            number INTEGER,
                            id_f FOREIGN KEY REFERENCES Table(ID)
            ) 
        """

        columns = {}
        names = []
        for k in kargs:
            names.append(k)
            v = kargs[k]
            if v is int:
                columns[k] = '%s %s' % (k, 'INTEGER ')
            elif v is str:
                columns[k] = '%s %s' % (k, 'TEXT ')
            elif v is time:
                columns[k] = '%s %s' % (k, 'TimeStamp DEFAULT CURRENT_TIMESTAMP')
            elif isinstance(v, int):
                columns[k] = '%s %s' % (k, 'INTEGER NOT NULL DEFAULT %d' % v)
            elif isinstance(v, str):
                columns[k] = '%s %s' % (k, 'VARCHAR(255) NOT NULL DEFAULT "%s"' % v)
            
            elif hasattr(v, '_table'):
                columns[k] = '%s %s' % (k, 'INTEGER, FOREIGN KEY (%s) REFERENCES %s(ID) ' % (k, v.__name__))
            else:
                raise TypeError("not supported Sql Type %s" % str(v))

        # extend some attribute
        for k in extend_options:
            columns[k] += extend_options[k]

        return self.create_table(table, *columns.values())
        

    def drop_table(self, table):
        cmd = 'DROP TABLE %s;' % table
        return self.run_cmd(cmd)

    def close(self):
        self.cu.close()
        self.con.close()

    def __del__(self):
        self.close()

    def _sql(self, k, v):
        if isinstance(v, int):
            return '{}={}'.format(k, v)
        elif isinstance(v, str):
            return '{}="{}"'.format(k, v)
        elif hasattr(v, '_table'):
            if hasattr(v, "ID"):
                return str(v.ID)
            return 'NULL'
        else:
            print(k, v)
            raise Exception("what type?")

    def select(self,table, *fields, **condition):
        

        cmd = "select {} from %s ".format(",".join(fields)) % table 
        if condition:
            cmd += 'where ' +  ' '.join([self._sql(*i) for i in condition.items()]) + ';'
        # print(cmd)
        self.cu.execute(cmd)
        for row in self.cu.fetchall():
            yield row

    def _sqls(self, i):
        if isinstance(i, int):
            return str(i)
        elif isinstance(i, str):
            return '"' + i + '"'
        elif hasattr(i, '_table'):
            if hasattr(i, "ID"):
                return str(i.ID)
            return 'NULL'
        elif i is time:
            return datetime.datetime.now()
        else:
            print(i)
            raise("error insert")

    def insert(self, table, insert_columns, *values, **kargs):
        values = [ self._sqls(i) for i in values ]
        sept = '?' if self.Type == 'sqlite' else '%s'
        # sept = '%s'
        values_tmp = ','.join([sept for i in range(len(values))])
        cmd = ("insert into %s ({order}) values ({values});" % table).format(order=','.join(insert_columns), values=values_tmp) 
        # print(cmd)
        try:
            self.cu.execute(cmd, values)
        except Exception as e:
            print(e)
            print(values)
            print(cmd)
            raise e

        return self.con.commit()

    def update(self, table, sets, **condition):
        cond = ' '.join([self._sql(*i) for i in condition.items()])
        updated = ' '.join([self._sql(*i) for i in sets.items()])
        cmd = '''UPDATE {table}  SET {updated}
        WHERE {condition} ;
        '''.format(table=table, updated=updated, condition=cond)
        return self.run_cmd(cmd)

    def delete(self, table, **condition):
        cond = ''
        if condition:
            cond = 'WHERE ' + ' '.join([self._sql(*i) for i in condition.items()])
        cmd = '''DELETE FROM {table}
        {condition} ;
        '''.format(table=table, condition=cond)

        return self.run_cmd(cmd)
    # def gen_table(self, *columns):
        # create_cmd = 'create '


class Table:
    """
    this is sql Table trans to class
    need set_handle(objhandle)
    """
    _table = 0
    _obj_handler = None

    def __init__(self, **kargs):
        # setattr(self.__class__, '_table', self.__class__.__name__)
        self.ID = None
        self.CreatedTime = None
        self._update = {}

        self._table = self.__class__.__name__
        # for k in self._columns():
        #     if k in kargs:
        #         setattr(self, k, kargs[k])
        #     else:
        #         setattr(self, k, self.__class__.__dict__[k])

        for k in kargs:
            if k in self._columns():
                setattr(self, k, kargs[k])
            elif k in ('ID', 'CreatedTime'):
                setattr(self, k, kargs[k])
            else:
                raise TypeError("table %s no such columns %s " %(self._table, k) )

        # for k in self._obj_columns():
        #     Obj = getattr(self.__class__, k)
        #     v = getattr(self, k)
        #     setattr(self, k, self._obj.find_one(Obj, ID=v))

    def __repr__(self):
        return self.__class__.__name__

    @classmethod
    def set_handle(cls, handler):
        cls._obj_handler = handler

    @classmethod
    def _columns(cls):
        return [i for i in cls.__dict__ if not i.startswith('__')]

    @classmethod
    def _obj_columns(cls):
        return [i for i in cls._columns() if hasattr(getattr(cls, i), '_table') ]

    def _dict(self):
        return {i:self[i] for i in self.__class__.__dict__ if not i.startswith('__')}

    @classmethod
    def _CREATE_ITEMS(cls):
        return {i:cls.__dict__[i] for i in cls.__dict__ if not i.startswith('__')}

    def __getitem__(self, k):
        v = getattr(self, k)
        if hasattr(getattr(self.__class__, k), '_table'):
            vv = getattr(self.__class__, k)
            return self._obj_handler.find_one(vv, ID=v)
        return v

    def __setitem__(self, k, v):
        self._update[k] = v

    def _save(self):
        if self.ID is None:
            return self._obj_handler.add(self)
        return self._obj_handler.save(self)

    def _delete(self):
        return self._obj_handler.delete(self)

    @staticmethod
    def hash( obj):
        return _hash(obj)

    @classmethod
    def _all(cls):
        return list(cls._obj_handler.find(cls))



class SqlObjectEngine:

    def __init__(self, **connect_option):
        self.sql = SqlEngine(**connect_option)

    def create(self, Obj):
        columns = Obj._CREATE_ITEMS()
        self.sql.create(Obj.__name__, **columns)

    def add(self, Obj_self):
        keys = []
        values = []
        _d = Obj_self._dict()
        for i in _d:
            # jump id and createdTime
            if not _d[i]:
                continue
            keys.append(i)
            values.append(_d[i])

        self.sql.insert(Obj_self.__class__.__name__, keys, *values)

    def _find(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        selected = ','.join(columns)
        return self.sql.select(obj.__name__, selected, **condition)

    def find_one(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        for row in self._find(obj, **condition):
            return obj(**dict(zip(columns, row)))
            

    def find(self, obj, **condition):
        columns = ['ID','CreatedTime'] +obj._columns()
        for row in self._find(obj, **condition):
            yield obj(**dict(zip(columns, row)))


    def save(self, obj_self):
        columns = ['ID','CreatedTime'] +obj_self._columns()
        if obj_self.ID != None:
            self.sql.update(obj_self._table, obj_self._update, ID=obj_self.ID)
            obj_self._update = {}

    def delete(self, obj_self):
        self.sql.delete(obj_self._table, ID=obj_self.ID)





from fabric.api import local
try:
    from bson import ObjectId
except ImportError as e:
    from bson import objectid as ObjectId
try:
    import time
    from QmongoHelper import Mongo
except ImportError as e:
    import sys
    print("you should install QmongoHelper from my Github", file=sys.stderr)

from redis import Redis

DB_NAME = 'local'


class Singleton(object):

    def __new__(cls, *args, **kargs):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls, *args, **kargs)
        return cls._instance



def db_add(doc, **data):
    db = Mongo(DB_NAME)
    db.insert(doc, **data)


def db_delete_one(_id, doc):
    db = Mongo(DB_NAME)
    return db.remove(doc, **{'_id':ObjectId(_id)})


def db_delete(_ids, doc):
    for _id in _ids:
        yield db_delete_one(_id, doc)


def db_search(info, doc, mon=None, year=None):
    if mon == None:
        mon = time.gmtime(time.time()).tm_mon
    db = Mongo(DB_NAME)
    res = db.find(doc, **{'time.year':year})
    if res == None:
        return []
    for i in res:
        if str(i).find(info) != -1 :
            yield i


def db_update(id, doc, **new):
    db = Mongo(DB_NAME)
    kargs = dict(**new)
    print({
            '$set':kargs
        })
    return db.update(doc, {'_id': ObjectId(id),},
        **{
            '$set':kargs
        })



class GotRedis(Singleton):

    def __init__(self, host='127.0.0.1', port=6379, db=0):
        self.host = "{ip}:{port}".format(ip=host, port=port)
        self.redis = Redis(host=host, port=port, db=db)

    def __call__(self, *args):
        if len(args) == 1:
            return self.redis.get(args[0])
        elif len(args) == 2:
            return self.redis.set(*args)
        elif len(args) == 3:
            return self.redis.hset(*args)

    def __add__(self, *key):
        if len(key) == 1:
            return self.redis.incr(key[0])
        elif len(key) == 2:
            return self.redis.sadd(*key)
        elif len(key) == 3:
            return self.redis.hincrby(*key)
        else:
            err("incr or  sadd or hincrby can be supported")
            return False

    def __getitem__(self, key):
        return self.redis.smembers(key)

    def __setitem__(self, member_p, member_c):
        return self.redis.sadd(member_p, member_c)

    def keys(self):
        return self.redis.keys()

def upload():
    local("cd ~/RMDB && git branch")
    args = input("db_name > ").split()
    db_name_pre, db_name = args if len(args) ==2 else ['',args[0]]
    local("cd ~/RMDB && git checkout %s %s && ./package.sh c && git add -A && git commit -m \"upload %s  \" && git push origin %s -f " % (db_name_pre ,db_name ,time.asctime(), db_name))


def load():
    local("cd ~/RMDB && git branch")
    args = input("db_name > ").split()
    db_name_pre, db_name = args if len(args) ==2 else ['',args[0]]
    local("cd ~/RMDB && git checkout %s %s && git pull origin %s &&  ./package.sh d  " % (db_name_pre, db_name,db_name))    





#!/usr/bin/env python3
import requests
import logging
import time
from lxml import etree
from .config import attr
from .config import check_url


def setLogging(file_name=time.asctime().replace(" ","_") + ".log", level=logging.WARNING):
    logging.basicConfig(
        level=level,
        format='%(asctime)s %(levelname)s %(message)s',
        datefmt='%a,%d,%Y',
        filename=file_name,
        filemode='a')

class Links_stack:
    got = set()
    session = None
    headers = None
    init = 1

    @staticmethod
    def init_session(init_url, headers=None):
        s = requests.Session()
        s.get(init_url, headers=headers)
        Links_stack.session = s
        return Links_stack.session

    @staticmethod
    def set_headers(headers):
        Links_stack.headers = headers


def get_headers(host, text=None, **kargs):
    headers_2 = {}
    if text:
        headers_2 = { i.split(':')[0]: i.split(':')[1] for i in text.split("\n")}

    raw_headers  = {
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'en-US,en;q=0.8',
        'Cache-Control': 'max-age=0',
        'Connection': 'keep-alive',
        'Host': host,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.106 Safari/537.36'
    }
    for k in headers_2:
        raw_headers[k] = headers_2[k]
        
    for k in kargs:
        raw_headers[k] = kargs[k]
    return raw_headers


def get_links(url, verify=False, init_url=None):
    if url != None:
        etr = None
        try:
            s = None

            if not Links_stack.session:
                s = Links_stack.init_session(init_url) if init_url else requests.Session()
                Links_stack.session = s
            else:
                s = Links_stack.session
            if Links_stack.init == 1:
                raw_response = s.get(url, verify=verify, headers=Links_stack.headers)
                print(Links_stack.headers)
                Links_stack.init = -1
            else:
                raw_response = s.get(url, verify=verify)
            print(attr.yellow %  '[%d]' % raw_response.status_code + attr.green % url)
            etr = etree.HTML(raw_response.content.decode(raw_response.encoding))
        except Exception as e:
            logging.fatal(attr.red % url)
            print(e)
            yield None

        if etr != None:
            logging.warn(attr.green % "load page:[got]" + url)
            for link in etr.xpath(attr.links):
                yield link.attrib['href']
        else:
            yield None
    else:
        yield None
    
def gen_points(head, url, ssl=False, exe=None):
    if url == None:
        yield None
    else:
        if not exe:
            for link in get_links(url, ssl):
                if link == None:
                    continue
                if link in Links_stack.got:
                    continue
                Links_stack.got.add(link)
                yield from gen_points(head, check_url(head, link, ssl), ssl=ssl, exe=exe)
        else:
            for link in exe.submit(get_links, url, ssl):
                if link == None:
                    continue
                if link in Links_stack.got:
                    continue
                Links_stack.got.add(link)
                yield from gen_points(head, check_url(head, link, ssl), ssl=ssl, exe=exe) 


import os
import sys
import json
import qlib.viewer.templates as templates

from copy import deepcopy

with open(os.path.join(os.environ['HOME'],".q_config")) as fp:
    BASE_URL = fp.read().strip()
    sys.path.append(BASE_URL)
from qlib.asyn import Exe
from qlib.net import to

_NodeClient__exe = Exe(6)
_GeoClient__exe = _NodeClient__exe

def callback(err, arg, result):
    if not err:
        pass
    else:
        print(err)


class ViewerBase:
    """
    type: 'node' for d3 force
          'tree' for d3 tree
    """
    def __init__(self, url, type):
        self.url = url
        self.template = getattr(templates, type)
        self.data = {}

    def _cmd(self, data):
        to(self.url, data=json.dumps(data), method='post')

class NodeClient(ViewerBase):

    def __init__(self, url):
        super(NodeClient, self).__init__(url, 'node')

    @__exe(callback=callback)
    def add_node(self, *ids, **options):
        node_data = self.template['node_data'].copy()
        node_data['msg']['nodes'] = [self._set_node(id, **options) for id in ids]
        self._cmd(node_data)
        return node_data

    @__exe(callback=callback)
    def add_link(self, s, *ts, **options):
        link_data = self.template['link_data'].copy()
        res = [{"from": s, "to": t} for t in ts]
        link_data['msg']['links'] = res
        self._cmd(link_data)
        return link_data

    def _set_node(self, id, **kargs):
        node = deepcopy(self.template['node'])
        node['id'] = id
        for k in kargs:
            node[k] = kargs[k]
        return node


class GeoClient(ViewerBase):
    
    def __init__(self, url):
        super(GeoClient, self).__init__(url, 'geo')

    @__exe(callback=callback)
    def msg(self, place, title, content, type="popover", **options):
        data = self.template['msg'].copy()
        data['msg']['title'] = title
        data['msg']['content'] = content
        data['msg']['place'] = place
        data['msg']['type'] = type
        self._cmd(data)
        return data

    @__exe(callback=callback)
    def control(self, place):
        data = self.template['control'].copy()
        data['msg']['place'] = place
        self._cmd(data)
        return data

    @__exe(callback=callback)
    def zoom(self, zoom):
        data = self.template['zoom'].copy()
        data['msg']['zoom'] = zoom
        self._cmd(data)
        return data

    @__exe(callback=callback)
    def mark(self, content, region=None):
        print(content)
        coor, msg = content.split("@")
        lan,lon = coor.split(",")
        data = self.template['mark'].copy()
        data['msg']['coor'] = [lan,lon]
        data['msg']['msg'] = msg
        if region:
            data['msg']['action'] = 'region'            
            r = {
                'action':'mark',
                'name':region
            }

            data['msg']['region'] = r
        self._cmd(data)
        return data

    @__exe(callback=callback)
    def link_two(self, f, t, time=8):
        data = self.template['sky_line'].copy()
        data['msg']['from'] = f
        data['msg']['to'] = t
        data['msg']['time'] = time
        self._cmd(data)
        return data


import matplotlib
import matplotlib.pyplot as plt
import matplotlib.animation as animation
matplotlib.style.use('ggplot')
import numpy as np
from matplotlib.lines import Line2D



class Drawer:
    """
    for drawer something 
    this is a power and abstract,
    put container in this , then ,just draw it or animation it
    """
    def __init__(self, container, x=[0,100], y=[0,256] ,ion=False, **options):
        if ion:
            plt.ion()
        
        self.fig, self.ax = plt.subplots()
        self.x = x
        self.y = y
        self.ax.set_xlim(x)
        self.ax.set_ylim(y)
        self.container = container(self.ax, **options)
        self.ani = None
        self.data_gen = None

    def update(self, *args, **kargs):
        return self.container.update(*args, **kargs)

    def show(self):
        plt.show()

    def draw(self):
        self.container.draw()

    def set_data(self, data):
        self.container.set_data(data)

    def set_data_gen(self, data_gen_func):
        self.data_gen = data_gen_func()

    def animal(self, interval=24):
        self.ani = animation.FuncAnimation(
            self.fig, 
            self.update,  #this function to update each step
            self.data_gen,  # this function will generator a data
            interval=interval, # set intervial time x frame each second
            blit=False)
        plt.show()


class Line:
    """
    ax: fig's axis
    dt: distance each step, default 1
    """
    def __init__(self, ax, dt=0.4, line_num=1,**options):
        self.ax = ax
        self.dt = dt
        for i in range(line_num):
            line = Line2D([0], [0])
            self.ax.add_line(line)
        if "style" in options:
            self.style = options['style']

    def update(self, ys):
        for i, y in enumerate(ys):
            line = self.ax.get_lines()[i]
            tdata = line.get_xdata()
            ydata = line.get_ydata()
            
            
            t = tdata[-1] + self.dt # new x data
            tdata.append(t)
            ydata.append(y)

            self.resize_panel(t, y) # resize the fig 

            line.set_data(tdata, ydata)
        return self.ax.get_lines()

    def resize_panel(self, t, y):
        lastt = t
        lasty = y
        change = False
        xlim_l, xlim_r = self.ax.get_xlim()
        ylim_l, ylim_r = self.ax.get_ylim()
        print(lastt, lasty)
        if lastt > xlim_r:  # reset the arrays

            change = True
            xlim_r *= 1.3
            self.ax.set_xlim(xlim_l, xlim_r)
            

        if lasty > ylim_r:
            change = True
            ylim_r *= 1.3
            self.ax.set_ylim(ylim_l, ylim_r)
        if lasty < ylim_l:
            change = True
            ylim_l -= ylim_r/3
            self.ax.set_ylim(ylim_l, ylim_r)


        if change:
            self.ax.figure.canvas.draw()
            change = False

    # def lim_change(self, style='append'):
    #     if style

    def draw(self):
        pass


class Kde:

    def __init__(self, ax):
        self.ax = ax

    def set_data(self, pd_data):
        self.data = pd_data

    def draw(self):
        self.data.plot(kind='kde')


# x = np.arange(0, 2*np.pi, 0.01)
# line, = ax.plot(x, np.sin(x))


# def animate(i):
#     line.set_ydata(np.sin(x + i/10.0))  # update the data
#     return line,


# # Init only required for blitting to give a clean slate.
# def init():
#     line.set_ydata(np.ma.array(x, mask=True))
#     return line,

# ani = animation.FuncAnimation(fig, animate, np.arange(1, 200), init_func=init,
#                               interval=25, blit=True)
# plt.show()



import concurrent
import time

from functools import wraps, partial
from collections import Iterable
from termcolor import cprint
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
from concurrent import futures

class Exe:
    Results = []

    def __init__(self, num, **options):
        self.callback = None

        for k in options:
            setattr(self, k, options[k])
        if isinstance(num, Iterable):
            self.thread_num = num[1]
            self.process_num = num[0]
        else:
            self.process_num = 1
            self.thread_num = num

        self.exe = ThreadPoolExecutor(max_workers=self.thread_num)

    def submit_result(self, funcs, *args, **kargs):
        futures = self.exe.submit(funcs, *args, **kargs)
        # Exe.Results.append( futures.result())
        return futures.result()

    def submit(self, funcs, *args, **kargs):
        futures = self.exe.submit(funcs, *args, **kargs)
        # Exe.Results.append( futures.result())
    
    def done(self, funcs, after_handle, *args, **kargs):
        futures = self.exe.submit(funcs, *args, **kargs)
        done_func = partial(self.on_done, after_handle)
        futures.add_done_callback(done_func)


    def map(self, func, args_iterable):
        for res in self.exe.map(func, args_iterable):
            yield res

    def __call__(self, timeout=9, callback=None):
        self.timeout = timeout
        if callback:
            self.callback = callback
        else:
            if not self.callback:
                self.callback = self.__callback

        def _wrap(func):

            @wraps(func)
            def __run(*args, **kargs):
                future = self.exe.submit(func, *args, **kargs)
                future.args = {"args": args, "kargs": kargs}
                return future.add_done_callback(self._background)
            return __run
        return _wrap

    def _background(self, future):
        err = future.exception()
        arg = future.args
        return self.callback(err, arg, future.result())

    def on_done(self, callback, future):
        return callback(*future.result())

    def __callback(self, exp, args, result):
        print(exp, args, result)

    def _t_run(self, seconds, function, *args, **kargs):
        time.sleep(seconds)
        return function(*args, **kargs)

    def timmer(self, time, function, *args, **kargs):
        self.submit(self._t_run, time, function, *args, **kargs)  
        



def __threads(n, func, *nargs):
    futures = set()
    results = set()
    with ThreadPoolExecutor(max_workers=n) as T:
        [futures.add(T.submit(func, arg)) for arg in nargs]
        for future in concurrent.futures.as_completed(futures):
            err = future.exception()
            if err is not None:
                cprint(err, "red")
            results.add(future.result())
    return results


def compute(func, args, process_num=2, thread_num=6, log=True):
    # queue = set()
    def get_args(num, args):
        s = []
        for i in args:
            s.append(i)
            if len(s) == num:
                yield s
                s = []

    futures = set()
    results = set()
    args_num = len(args) / process_num
    print(process_num, thread_num)
    if log:
        cprint('==> start %s [%d]' % (time.asctime(), args_num), 'cyan')

    with ProcessPoolExecutor(process_num) as P:
        for arg in get_args(args_num, args):
            if log:
                print(arg)
            futures.add(P.submit(__threads, thread_num, func, *arg))
        for f in concurrent.futures.as_completed(futures):
            for e in f.result():
                # if generate:
                #     yield e
                results.add(e)
                # pass
            if log:
                cprint('end <== %s' % time.asctime(), 'blue')

        # if not generate:
            return results



import select
import errno
from collections import defaultdict
from functools import partial
from termcolor import cprint, colored

S_NULL = 0x00
S_IN = 0x01
S_OUT = 0x02
S_ERR = 0xff

err_log = lambda x, y: print("[%s] {} {}".format(x, y) % "err")

class ELoop:
    """
    this can hadnle a obj by run handle(fd, mode) 
    """
    NULL = 0x00
    IN = 0x01
    OUT = 0x02
    ERR = 0xff

    def __init__(self, errlog=err_log):
        self._in_fd = set()
        self._out_fd = set()
        sefl._excep_fd = set()
        self._events = defaultdict(lambda:S_NULL)
        self.err_log = err_log
        self._stopping = False

    def _install(self, fd, mode):
        if mode & S_IN:
            self._in_fd.add(fd)
        if mode & S_OUT:
            self._out_fd.add(fd)
        if mode & S_ERR:
            self._excep_fd.add(fd)

    def _uninstall(self, fd):
        if fd in self._in_fd:
            self._in_fd.remove(fd)
        if fd in self._out_fd:
            self._out_fd.remove(fd)
        if fd in self._excep_fd:
            self._excep_fd.remove(fd)

    def _modify(self, fd, mode):
        self._uninstall(fd)
        self._install(fd, mode)

    def _select(self, timeout):
        r, w, x = select.select(self._in_fd, self._out_fd, self._excep_fd, timeout)
        for fds in [(r, S_IN), (w, S_OUT), (x, S_ERR)]:
            for fd in fds[0]:
                self._events[fd[0]][0] |= fds[1]

        return self._events.items()

    def add(self, f, mode, handler_name):
        fd = f.fileno()
        self._install(fd, mode)
        self._events[fd] = (f, handler_name)

    def change(self, fd, mode=None, handler=None):
        sock = fd.fileno()
        self._modify(fd, mode)
        old = self._events[fd]
        new = list(old)
        if mode:
            new[0] = mode
        if handler:
            new[1] = handler
        self._events[fd] = tuple(new)

    def stop(self):
        self._stopping = True

    def loop(self, timeout=None):
        events = []
        err_log_no_method = partial(err_log, "callback")
        while self._stopping:
            try:
                events = [(fd, sock_handler[0], sock_handler[1])  for fd , sock_handler in self._select(timeout)]
            except (OSError, IOError) as e:
                if error(e) in (errno.EPIPE, errno.EINTR):
                    self.err_log("select", e)
                else:
                    self.err_log("error", e)
                    continue
            for fd,  sock, handler in events:
                if handler:
                    try:
                        handler(sock)

                    except (OSError, IOError) as e:
                        err_log("event handling", e)
                else:
                    err_log_no_method("no event method can be callback")



    
    def close(self):
        self._stopping = True
        pass

    def __del__(self):
        self.close()


def error(e):
    """
    get errorno from Exception
    """
    if hasattr(e, 'errno'):
        return e.errno
    elif e.args:
        return e.args[0]
    else:
        return None



__all__ = [
	"attr",
	"en_points",
	"setLogging",
	"Links_stack",
	"get_headers",
]

from .config import attr   
from .get_point import gen_points
from .get_point import setLogging
from .get_point import Links_stack
from .get_point import get_headers





def gen_urls(f, default="http://"):
    with open(f) as fp:
        for line in fp:
            t = check_uri(line, default)
            if t:
                yield t


def check_uri(line, default="http://"):
    l = line.strip()
    if len(l.split()) != 1:
        return False

    if not l.startswith(default):
        return default + l
    else:
        return l

def url_to_db(db, url):
    rs = re.findall(r'(?:http\://.+?)(/.+)', url)
    if rs:
        db.insert("urls",**{"uri":rs[0]})



#!/usr/bin/env python3
import json


class JsonFile:

    def __init__(self, f):
        with open(f) as fp:
            self.json = json.loads(fp.read())


class JsonData:

    def __init__(self):
        self.json = None


class TreeD:
    """
        the raw data must a dict and has 'id' and 'parent'
    """
    table = dict()
    table_keys = set()
    root = set()
    tmp_children = dict()
    tag = False

    def __init__(self, node):
        self.children = []
        self.id = node['id']
        self.parent_string = node['parent']
        self.parent = None
        self.attr = node
        self.load()

    def load(self):
        TreeD.tag = True
        if self.id in TreeD.table_keys:
            self = TreeD.get(self.id)
            return

        TreeD.table[self.id] = self
        TreeD.table_keys.add(self.id)

        # if node is root , add this to root
        if self.parent_string == self.id:
            TreeD.root.add(self)

        # collect children in tmp place
        if self.id in TreeD.tmp_children:
            self.children = TreeD.tmp_children.pop(self.id)

        # if this node's father exists ,
        # this node can collected by father directly
        if self.parent_string in TreeD.table_keys:
            TreeD.table[self.parent_string].child(self)
            self.parent = TreeD.table[self.parent_string]
        # if this node's father not exists now,
        # put node to a tmp place
        elif self.parent_string in TreeD.tmp_children:
            TreeD.tmp_children[self.parent_string].append(self)
        # if there is not any tmp place ...., created one
        else:
            TreeD.tmp_children[self.parent_string] = [self]

    def child(self, node):
        if node.id == self.id:
            return
        print(node.id, "--> ", self.id)
        self.children.append(node)

    def keys(self):
        return TreeD.table_keys

    def to_json(self):
        if self.children:
            c = {
                "id": self.id,
                "attr": self.attr,
                "children": [child.to_json() for child in self.children],
            }
            return c
        else:
            return {
                "id": self.id,
                "attr": self.attr,
                "size": len(self.id)
            }

    def __getitem__(self, k):
        return TreeD.get(k)

    def __call__(self):
        if not self.parent:
            return self
        else:
            return self.parent()

    def __str__(self):
        return self.id

    @staticmethod
    def tmp_to_root():
        for i in TreeD.tmp_children:
            t = TreeD({
                "id": i,
                "link": "http://" + i + "/",
                "parent": i,
                "title": i,
            })
            t.children = TreeD.tmp_children[i].copy()
            TreeD.root.add(t)
        TreeD.tmp_children = None

    @staticmethod
    def get_root():
        return TreeD.root

    @staticmethod
    def get(key):
        return TreeD.table[key]

    @staticmethod
    def clear():
        TreeD.table = dict()
        TreeD.table_keys = set()
        TreeD.root = set()
        TreeD.tmp_children = dict()


class Tree(JsonFile):

    def __init__(self, f):
        super(Tree, self).__init__(f)

        self.root = {
            "name": "root",
            "children": [],
        }
        self.__data = TreeD
        # if self.__data.tag:
        #     self.__data.clear()
        self.__parse()

    def __parse(self):
        for x in self.json:
            TreeD(x)

        self.__data.tmp_to_root()

    def __getitem__(self, k):
        return self.__data.table[k]

    def keys(self):
        return self.__data.table_keys

    def __call__(self):
        return self.__data.root

    def to_json(self, file=None):
        c = [subroot.to_json() for subroot in self()]
        self.root["children"] = c
        self.size = len(c)
        if not file:
            return self.root
        with open(file, "w") as fp:
            fp.write(json.dumps(self.root))



from .__db import db_add, db_delete, upload, load, db_update, db_search, db_delete_one, DB_NAME, GotRedis

__all__ = [
    'db_add', 'db_delete', 'db_update', 'upload', 'load', 'db_search', 'db_delete_one', 'DB_NAME', 'GotRedis', 'Sql'
]





from setuptools import setup, find_packages


setup(name='Mroylib',
    version='0.2',
    description='some lib',
    url='https://github.com/Qingluan/Q.git',
    author='Qing luan',
    author_email='darkhackdevil@gmail.com',
    license='MIT',
    zip_safe=False,
    packages=find_packages(),
    install_requires=['Qtornado','redis','pymysql','fabric3','requests','lxml','bs4','termcolor','bson','simplejson'],

)





#!/usr/bin/env python3
import requests
from lxml import etree
from concurrent.futures import ThreadPoolExecutor


def got(url):
    file_url = url.replace("/", "_")
    with open(file_url, "wb") as tar: 
        tar.write(requests.get(url, verify=False).content)


def read_list(f):
    with open(f) as fp:
        for line in fp:
            yield line


def download(lst):
    with ThreadPoolExecutor(10) as exe:
        wc = 0
        for link in lst:
            print("sub : %s %d" % (link, wc),end="\r")
            exe.submit(got, link)
            wc += 1
        exe.shutdown()



import os, sys, time
import json
from queue import Queue, Empty
from termcolor import cprint
from urllib.parse import urljoin
from functools import wraps
from collections import defaultdict

sys.path.append(os.getenv("Q_PATH"))
from qlib.asyn import Exe, futures
from qlib.net import to
from qlib.graphy import random_choice
from qlib.spide.agents import AGS
from qlib.spide.web import Handler
from qlib.log import LogControl


LogControl.LOG_LEVEL = LogControl.OK
LogControl.LOG_LEVEL |= LogControl.WRN
LogControl.LOG_LEVEL |= LogControl.INFO
LogControl.LOG_LEVEL |= LogControl.FAIL

info = LogControl.info
err = LogControl.err
fail = LogControl.fail
ok = LogControl.ok
wrn = LogControl.wrn


class SqlHandler(Handler):

    def __init__(self, depth=1, *args, **kargs):
        super(SqlHandler, self).__init__(*args, **kargs)
        self.depth = depth
        self.no_depth = 0
        self.sql_filter = Queue()

    def on_start(self, url, parser):
        self.count = 0
        if parser is None:
            return

        for i in parser.xpath("//a[@href!='']"):
            self.ready_link(i.attrib['href'])

        self.run(self.on_detail, limit_wait=self.try_times)

    def on_detail(self, url, parser):
        # LogControl.ok(url)
        # info("parent: ", url, txt_attr=['bold'])
        if parser is None:
            return

        for i in parser.xpath("//a[@href!='']"):
            link = i.attrib['href']
            if self.no_depth < self.depth:
                # info("put", link)
                self.ready_link(link)
                if self.filter(link):
                    ok("find", link)
                    self.sql_filter.put(link)
            else:
                self.dead()

            if link not in self.reach_url:
                ok('  [%d]' % self.count,end='\r')
                sys.stdout.flush()
                self.count += 1
        self.no_depth += 1

class SqlmapApi:
    """
    Class Task(Table):
        task_id: "some hash id"
        url: str,
        status: 'not running'
        finishtime: time

@get("/task/new")  
@get("/task/<taskid>/delete")  
@get("/admin/<taskid>/list")  
@get("/admin/<taskid>/flush")  
@get("/option/<taskid>/list")  
@post("/option/<taskid>/get")  
@post("/option/<taskid>/set")  
@post("/scan/<taskid>/start")  
@get("/scan/<taskid>/stop")  
@get("/scan/<taskid>/kill")  
@get("/scan/<taskid>/status")  
@get("/scan/<taskid>/data")  
@get("/scan/<taskid>/log/<start>/<end>")  
@get("/scan/<taskid>/log")  
@get("/download/<taskid>/<target>/<filename:path>")  
    """

    TAMPER = [
        'apostrophemask',
        'apostrophenullencode',
        'appendnullbyte',
        'base64encode',
        'between',
        'bluecoat',
        'chardoubleencode',
        'charencode'
    ]

    def __init__(self, id=None, db=None, host="127.0.0.1", port="8775", thread_num=2):
        self.host = host
        self.port = port
        self.exe = Exe(thread_num)
        self.db = db
        self.target = 'http://{host}:{port}'.format(host=host, port=port)
        self.test_url = set()
        self.injectable = defaultdict(lambda:None)
        self.id = id
        self.connect = False

        if self.id is None:
            self.create_task()

    def on_result(self, t, v):
        if t == 'new':
            self.id = v['taskid']
            if self.db is not None:
                self.db.insert("Task", ['task_id', 'url', 'status'], self.id, 'None', 'not running')

        elif t == 'data':
            info(v)
            if v['data']:
                if self.db:
                    self.db.update("Task", )
                else:
                    ok(v['data'])
                    self.injectable = True

        elif t == "status":
            info(v[u'status'])
        elif t == 'start':
            info(v['success'])
        elif t == 'set':
            ok('\ninit options')
        elif t == 'kill':
            fail(v)
        elif t == 'stop':
            wrn(v)
        elif t == 'list':
            for k in v['options']:
                ok(k, v['options'][k])
        elif t == 'task':
            info(v)
        elif t == 'log':
            for msg in v['log']:
                info(msg)

    def handle(self, tag, cmd, **kargs):
        return tag, to(urljoin(self.target, cmd), **kargs).json()

    def create_task(self):
        try:
            self.exe.done(self.handle, self.on_result, 'new', 'task/new')
            self.connect = True
        except Exception as e:
            err("check sqlmapapi server in -> ",self.host, self.port)
            self.connect = False

    def delete_task(self):
        self.cmd('task', 'delete')
        self.id = None
        # self.injectable = False
        # self.test_url = None

    def scan_cmd(self, cmd):
        # self.exe.done(self.handle, self.on_result, cmd, 'scan/{id}/{cmd}'.format(id=self.id, cmd=cmd))
        self.cmd('scan', cmd)

    def task_cmd(self, cmd):
        self.cmd('task', cmd)

    def option(self, cmd):
        self.cmd('option', cmd)

    def cmd(self, tag, cmd):
        if self.connect:
            self.exe.done(self.handle, self.on_result, cmd, '{tag}/{id}/{cmd}'.format(tag=tag, id=self.id, cmd=cmd))
        else:
            wrn("not connect")

    def scan(self, url, **options):
        '''
        @url: target url
        @options: {
                    'tamper': True,
                    'smart': True,
                    'delay': 1
                }
        '''
        if 'tamper' in options:
            options['tamper'] = SqlmapApi.TAMPER

        self.test_url.add(url)
        self.injectable[url] = self.id
        if options:
            data = json.dumps(options)
            self.exe.done(
                self.handle,
                self.on_result,
                'set',
                'option/{id}/set'.format(id=self.id),
                data=data,
                method='post',
                headers={'Content-Type': 'application/json'})

        data = json.dumps({'url': url})
        self.exe.done(
            self.handle,
            self.on_result, 
            'start',
            'scan/{id}/start'.format(id=self.id),
            data=data,
            method='post',
            headers={'Content-Type': 'application/json'})

    def status(self):
        self.scan_cmd('status')

    def log(self):
        self.scan_cmd('log')

    def stop(self, id=None):
        self.scan_cmd("stop")










        



#!/usr/bin/env python3
import os
from subprocess import Popen ,PIPE
import pexpect
from .softtables import table
from .softtables import toptbale


def _shell_do(prefix,content,passwd=None):
    _content = " \"{}\" ".format(content)
    result = None
    res = 127
    if passwd:
        result = pexpect.spawn(prefix + _content,timeout=600)
        result.expect('password:')
        result.sendline(passwd+'\r')
        res = result.read().decode()
        if "not" in res.split():

            print("Err  ")
            return 1
        print("True ")
        return 0
        

    else:
        result = Popen(prefix + _content,stdout=PIPE,shell=True)
        res = result.wait()
        print (_content)
        out,err= result.communicate('qingluan\n')

        print(out)
#    result.stdout.readline()
    return res

def _shell_install(prefix,content,passwd=None):
    _content = " \"{}\" ".format(content)
    result = pexpect.spawn(prefix + _content,timeout=600)
    result.expect('password:')
    result.sendline(passwd)
    result.expect('y')
    result.sendline("yes")
    print(result.read().decode())


def _remote_copy(prefix,filename,path,passwd=None):
    _content = "scp {} {}  {}".format(prefix,filename,path)

    if passwd:
        result = pexpect.spawn(_content,timeout=600)
        result.expect('password:')
        result.sendline(passwd+"\r")
        res = result.readlines()
        print(res)
        for line in res:
            if b"not" in line.split():
                return 127
        return 0
    else:
        result = Popen(_content,stdout=PIPE,shell=True)
    return result.wait()




def checkout_soft(login_info,name,passwd=None):
    cmd = "{} -h ".format(name)

    user, ip,port = login_info
    ssh_cmd = "ssh -p {} {}@{}".format(port,user,ip)
    res = _shell_do(ssh_cmd,cmd,passwd)

    if res == 0: 
        return True 
    return False


def checkout_softs(login_info,names,passwd=None):
    return {name:checkout_soft(login_info,name,passwd) for name in names}


def top_install_soft(login_info,names,passwd=None):
    sys_version = checkout_softs(login_info,["apt-get","yum"],passwd)
    install_soft = "apt-get"
    for k in ["apt-get","yum"]:
        if sys_version[k]:
            install_soft = k

    user,ip,port = login_info
    unstall_softs = [toptbale[soft] for soft in names  ] 
    cmds = "{} install {}".format(install_soft," ".join(unstall_softs))
    ssh_cmd = "ssh -p {} {}@{} ".format(port,user,ip)
    res = _shell_install(ssh_cmd,cmds,passwd)
    return res



def normal_install_soft(login_info,softtables,passwd=None):
    cmds = " && ".join([table[soft] for soft in softtables ] )
    user,ip,port = login_info
    ssh_cmd = "ssh -p {} {}@{} ".format(port,user,ip)

    res = _shell_install(ssh_cmd,cmds,passwd)
    return res

def get_uninstall_softs(softs):
    uninstall = [ soft for soft in softs if not softs[soft] ]
    
    normal = [soft for soft in uninstall if soft in table]
    top  = list(set(uninstall) - set(normal))

    return [top,normal]

    
def copy_file(login_info,f,path,passwd=None):
    user,ip,port = login_info
    prefix = "-P  {}".format(port)
    path = " {}@{}:{} ".format(user,ip,path)
    
    return _remote_copy(prefix,f,path,passwd)

def run_shdowsocks(login_info,passwd=None):
    user,ip,port = login_info

    ssh_cmd = "ssh -p {} {}@{} ".format(port,user,ip)
    cmds = "ssserver -c ./default_config.json -d start "
    _shell_do(ssh_cmd,cmds,passwd)

def get_shadow_status(login_info,passwd=None):
    user,ip,port = login_info

    ssh_cmd = "ssh -p {} {}@{} ".format(port,user,ip)
    cmds = "ps aux | grep sss | egrep -v \"(grep|egrep)\""
    return _shell_do(ssh_cmd,cmds,passwd)
